'use client';
import { z } from 'zod';
import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import { getProfile, getSignedUrl, updateProfile, uploadFile } from '@/lib/actions';
import { useState, useEffect, useRef } from 'react';
import type { Profile } from '@/utils/supabase';
import {
  Avatar,
  AvatarFallback,
  AvatarImage,
  Button,
  Card,
  CardContent,
  CardDescription,
  CardHeader,
  CardTitle,
  Form,
  FormControl,
  FormDescription,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
  Input,
  Separator,
} from '@/components/ui';
import { toast } from 'sonner';
import { Pencil, User } from 'lucide-react'

const formSchema = z.object({
  full_name: z.string().min(5, {
    message: 'Full name is required & must be at least  5 characters.'
  }),
  email: z.string().email(),
});

const ProfilePage = () => {
  const [profile, setProfile] = useState<Profile | undefined>(undefined);
  const [avatarUrl, setAvatarUrl] = useState<string | undefined>(undefined);
  const [isLoading, setIsLoading] = useState(true);
  const [isUploading, setIsUploading] = useState(false);
  const fileInputRef = useRef<HTMLInputElement>(null);


  const form = useForm<z.infer<typeof formSchema>>({
    resolver: zodResolver(formSchema),
    defaultValues: {
      full_name: '',
      email: '',
    },
  });

  useEffect(() => {
    const fetchProfile = async () => {
      try {
        setIsLoading(true);
        const profileResponse = await getProfile();
        if (!profileResponse.success)
          throw new Error('Profile response unsuccessful');
        setProfile(profileResponse.data);
        form.reset({
          full_name: profileResponse.data.full_name ?? '',
          email: profileResponse.data.email ?? '',
        });
      } catch (error) {
        setProfile(undefined);
      } finally {
        setIsLoading(false);
      }
    };
    fetchProfile().catch((error) => {
      console.error('Error getting profile:', error);
    });
  }, [form]);

  useEffect(() => {
    const getAvatarUrl = async () => {
      if (profile?.avatar_url) {
        try {
          const response = await getSignedUrl({
            bucket: 'avatars',
            url: profile.avatar_url,
          });
          
          if (response.success) {
            setAvatarUrl(response.data);
          }
        } catch (error) {
          console.error('Error getting signed URL:', error);
        }
      }
    };
    getAvatarUrl().catch((error) => {
      toast.error(error instanceof Error ? error.message : 'Failed to get signed avatar url.');
    });
  }, [profile]);

  const handleAvatarClick = () => {
    fileInputRef.current?.click();
  };

  const handleFileChange = async (e: React.ChangeEvent<HTMLInputElement>) => {
    const file = e.target.files?.[0];
    if (!file) 
      throw new Error('No file selected');

    try {
      setIsUploading(true);

      const fileExt = file.name.split('.').pop();
      const fileName = `${Date.now()}-${profile?.id ?? Math.random().toString(36).substring(2,15)}.${fileExt}`;

      const uploadResult = await uploadFile({
        bucket: 'avatars',
        path: fileName,
        file,
        options: {
          upsert: true,
          contentType: file.type,
        },
      });
      if (!uploadResult.success)
        throw new Error(uploadResult.error ?? 'Failed to upload avatar');

      const updateResult = await updateProfile({
        avatar_url: uploadResult.data,
      });

      if (!updateResult.success)
        throw new Error(updateResult.error ?? 'Failed to update profile');

      setProfile(updateResult.data);
      toast.success('Avatar updated successfully.')

    } catch (error) {
      toast.error(error instanceof Error ? error.message : 'Failed to uploaad avatar.');
    } finally {
      setIsUploading(false);
      if (fileInputRef.current) fileInputRef.current.value = '';
    }
  };

  const getInitials = (name: string) => {
    return name
      .split(' ')
      .map((n) => n[0])
      .join('')
      .toUpperCase();
  }

  const onSubmit = async (values: z.infer<typeof formSchema>) => {
    try {
      setIsLoading(true);
      const result = await updateProfile({
        full_name: values.full_name,
        email: values.email,
      });

      if (!result.success) {
        throw new Error(result.error ?? 'Failed to update profile');
      }
      setProfile(result.data);
      toast.success('Profile updated successfully!');
    } catch (error) {
      toast.error(error instanceof Error ? error.message : 'Failed to update profile.');
    } finally {
      setIsLoading(false);  
    }
  };

  if (profile === undefined)
    return (
      <div className='flex p-5 items-center justify-center'>
        <h1>Unauthorized</h1>
      </div>
  );

  return (
    <Card className='p-8'>
      <CardHeader className='pb-2'>
        <CardTitle className='text-2xl'>
          {profile?.full_name ?? 'Profile'}
        </CardTitle>
        <CardDescription>
          Manage your personal information & how it appears to others.
        </CardDescription>
      </CardHeader>
      <CardContent>
        {isLoading  && !profile ? (
          <div className="flex justify-center py-8">
            <div className="animate-pulse text-center">
              <div className="h-24 w-24 rounded-full bg-gray-200 mx-auto mb-4"></div>
              <div className="h-4 w-48 bg-gray-200 mx-auto"></div>
            </div>
          </div>
        ) : (
          <div className="space-y-8">
            <div className="flex flex-col items-center">
              <div className="relative group cursor-pointer mb-4" onClick={handleAvatarClick}>
                <Avatar className="h-32 w-32">
                  {avatarUrl ? (
                    <AvatarImage src={avatarUrl} alt={profile.full_name ?? 'User'} />
                  ) : (
                    <AvatarFallback className="text-2xl">
                      {profile?.full_name 
                        ? profile.full_name.split(' ').map(n => n[0]).join('').toUpperCase() 
                        : <User size={32} />}
                    </AvatarFallback>
                  )}
                </Avatar>
                <div className="absolute inset-0 rounded-full bg-black/0 group-hover:bg-black/50 transition-all flex items-center justify-center">
                  <Pencil className="text-white opacity-0 group-hover:opacity-100 transition-opacity" size={24} />
                </div>
                <input
                  ref={fileInputRef}
                  type="file"
                  accept="image/*"
                  className="hidden"
                  onChange={handleFileChange}
                  disabled={isUploading}
                />
              </div>
              {isUploading && (
                <div className="text-sm text-gray-500">Uploading...</div>
              )}
              <p className="text-sm text-gray-500 mt-2">
                Click on the avatar to upload a new image
              </p>
            </div>
            <Separator />
            <Form { ...form}>
              <form
                onSubmit={form.handleSubmit(onSubmit)}
                className='space-y-6'
              >
                <FormField
                  control={form.control}
                  name='full_name'
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel>Full Name</FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormDescription>
                        Your public display name.
                      </FormDescription>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <FormField
                  control={form.control}
                  name='email'
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel>Email</FormLabel>
                      <FormControl>
                        <Input {...field} />
                      </FormControl>
                      <FormDescription>
                        Your email address associated with your account.
                      </FormDescription>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                <div className="flex justify-center">
                  <Button type='submit' disabled={isLoading}>
                    {isLoading ? 'Saving...' : 'Save Changes'}
                  </Button>
                </div>
              </form>
            </Form>
          </div>
        )}
      </CardContent>
    </Card>
  );
};
export default ProfilePage;
